// Sample_Prn.cpp

#include "stdafx.h"
#include "resource.h"
#include "PrintCE.h"

#define APP_NAME	_T("Sample_PDC")
#define ID_PRINT	500
#define ID_SILENT	501
#define ID_LANG		502
#define ID_EXIT		503

HINSTANCE			hInst;
HWND				hWnd;
HWND				hBtn[4] = { NULL };
int					cur_lang = 0; // english

LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL				InitInstance(HINSTANCE);
void				OnSize();
void				OnPrint(BOOL silent);
void				OnLang();


int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
 	MSG msg;

	hInst = hInstance;

	// Perform application initialization:
	if(!InitInstance(hInstance)) 
		return FALSE;

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

BOOL InitInstance(HINSTANCE hInstance)
{
	WNDCLASS	wc;

	// register class
    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= NULL;
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= APP_NAME;

	RegisterClass(&wc);

	// create window
	hWnd = CreateWindow(APP_NAME, APP_NAME, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	if(!hWnd)
		return FALSE;

	// create buttons
	hBtn[0] = CreateWindow(_T("Button"),_T("Print"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_PRINT,hInstance,NULL);
	hBtn[1] = CreateWindow(_T("Button"),_T("Silent Print"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_SILENT,hInstance,NULL);
	hBtn[2] = CreateWindow(_T("Button"),_T("Language"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_LANG,hInstance,NULL);
	hBtn[3] = CreateWindow(_T("Button"),_T("Exit"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_EXIT,hInstance,NULL);

	if(!hBtn[0])
		return FALSE;

	// resize
	OnSize();

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC			hdc;
	PAINTSTRUCT ps;

	switch(message)  {
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case ID_PRINT:
				   OnPrint(FALSE);
				   break;
				case ID_SILENT:
					OnPrint(TRUE);
				   break;
				case ID_LANG:
					OnLang();
					break;
				case ID_EXIT:
				   DestroyWindow(hWnd);
				   break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			OnSize();
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

void OnSize()
{
	RECT	rc;
	int		width,height,space;

	if(!hBtn[0])
		return;

	GetClientRect(hWnd,&rc);
	width = (rc.right - rc.left)/2;
	space = (rc.bottom - rc.top)/13;
	height = space * 2;

	MoveWindow(hBtn[0],width/2,space,width,height,TRUE);
	MoveWindow(hBtn[1],width/2,space*2+height,width,height,FALSE);
	MoveWindow(hBtn[2],width/2,space*3+height*2,width,height,FALSE);
	MoveWindow(hBtn[3],width/2,space*4+height*3,width,height,FALSE);
}

void OnLang()
{
	RECT	rc;
	HMENU	hMenu;
	int		id;

	GetClientRect(hWnd,&rc);

	hMenu = CreatePopupMenu();

	AppendMenu(hMenu,MF_STRING,1,_T("English"));
	AppendMenu(hMenu,MF_STRING,2,_T("Russian"));
	AppendMenu(hMenu,MF_STRING,3,_T("French"));
	AppendMenu(hMenu,MF_STRING,4,_T("Portuguese"));
	AppendMenu(hMenu,MF_STRING,5,_T("Italian"));
	AppendMenu(hMenu,MF_STRING,6,_T("Dutch"));

	CheckMenuItem(hMenu,cur_lang+1,MF_BYCOMMAND|MF_CHECKED);

	id = TrackPopupMenu(hMenu,TPM_BOTTOMALIGN|TPM_RIGHTALIGN|TPM_RETURNCMD,(rc.right-rc.left)/2,(rc.bottom-rc.top)/2,0,hWnd,0);

	DestroyMenu(hMenu);

	if(id == 0)
		return;

	cur_lang = id - 1;
}

void OnPrint(BOOL silent)
{
	double		y = 0.0;
	HBITMAP		hBmp;

	// load print library
	if(!LoadPrintLib())
		return;

	// library initialization (licence key is transfered as a parameter)
	prnInit(_T("My license key"));

	// set language of the user interface
	prnSetLanguage(cur_lang);

	// switch measuring units to inches
	prnSetMeasureUnit(kInches);

	if(silent) {
		// don't show print setup dialog, just print to file 'silent_out.prn' for 'HP PCL' printer
		PRN_INFO	info;

		info.printer = 0;						// HP PCL
		info.port = 9;							// output to file
		info.portrait = TRUE;					// portrait mode
		info.color_mode = 2;					// CMY colors
		info.draft_mode = FALSE;				// no draft
		info.paper = 0;							// 'A4' paper for HP printer (see "PrintCE documentation")
		info.paper_size.cx = 0;					// used only for 'custom paper'
		info.paper_size.cy = 0;					// used only for 'custom paper'
		info.start_page = info.end_page = -1;	// to print all pages
		info.user_margins.top = info.user_margins.bottom = info.user_margins.left = info.user_margins.right = 0; // zero margins
		// set output file path
		HKEY	hKey;
		LPCTSTR	path = _T("\\silent_out.prn");
		if(ERROR_SUCCESS == RegOpenKeyEx(HKEY_CURRENT_USER,_T("Software\\IneSoft\\PrintCE"),0,NULL,&hKey)) {
			RegSetValueEx(hKey,_T("FilePath"),0,REG_SZ,(BYTE*)path,sizeof(TCHAR)*(_tcslen(path) + 1));
			RegCloseKey(hKey);
		}
		// silent setup
		if(!prnSilentPrintSetup(hWnd,&info)) {
			prnUnInit();
			UnloadPrintLib();
			MessageBox(hWnd,_T("prnSilentPrintSetup failed !"),APP_NAME,MB_OK);
			return;
		}
		// disable progress dialog box
		prnSetSilentMode(TRUE);
	} else {
		// show print setup dialog
		if(!prnSetupDlg(hWnd)) {
			prnUnInit();
			UnloadPrintLib();
			return;
		}
	}

	// set margins
	prnSetLeftMargin(1.0);
	prnSetRightMargin(1.0);
	prnSetTopMargin(0.7);
	prnSetBottomMargin(0.7);

	// start printing document
	prnStartDoc();

	// start printing page
	prnStartPage();

	// print heading with font of 14 points in height, bold 
	prnSetFontSize(14.0);
	prnSetFontBold(TRUE);
	prnDrawAlignedText(_T("PrintCE for Pocket PC"),prnGetPageWidth()/2,y,hCenter,vTop);
	y += prnGetTextHeight(_T("PrintCE for Pocket PC"));

	// print bitmap
	hBmp = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_BITMAP1)); // load bitmap from resource
	if(hBmp) {
		prnDrawBitmap(hBmp,0.0,y,0.8,0.0,TRUE);
		DeleteObject(hBmp);
		y += 0.9;
	}
	
	// set blue line colour
	prnSetLineColor(RGB(0,0,255));

	// set line width of 0.05 inches
	prnSetLineWidth(0.05);

	// print 3 blue lines of 2, 1.5 and 1 inches in length
	prnDrawLine(0,y,2.0,y);
	y += 0.3;
	prnDrawLine(0,y,1.5,y);
	y += 0.3;
	prnDrawLine(0,y,1.0,y);
	y += 0.3;
	
	// set yellow line colour
	prnSetLineColor(RGB(255,255,0));
	
	// set green fill colour
	prnSetFillColor(RGB(0,255,0));

	// print rectangle of 2 inches in length and 0.5 inch in width
	prnDrawRect(0.0, y, 2.0, y + 0.5);

	// set line width of 0.01 inches
	prnSetLineWidth(0.01);

	// set black line colour
	prnSetLineColor(RGB(0,0,0));

	// set red fill colour
	prnSetFillColor(RGB(255,0,0));

	// print ellipse inside the rectangle
	prnDrawEllipse(0.5, y + 0.05, 1.5, y + 0.45);
	y += 0.9;

	// print text with 9 points font 
	prnSetFontBold(FALSE);
	prnSetFontSize(9.0);
	prnDrawText(_T("http:\\www.inesoft.com"),0,y);
	
	// end printing document
	prnEndDoc();

	// library deinitialization
	prnUnInit();

	// Unload print library
	UnloadPrintLib();

	// show message for user
	if(silent)
		MessageBox(hWnd,_T("Silent printing done !"),APP_NAME,MB_OK|MB_ICONINFORMATION);
}